// UART Library for PIC 16F
// SDCC version / small device c compiler
// written by Jean-Pierre MANDON 2006 jp.mandon@free.fr
// www.pictec.org
/*
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
*/

// SPBRG value for FOSC internal 8 Mhz

#define B2400_i8         207
#define B9600_i8         51
#define B19200_i8        25
#define B28800_i8	 16
#define B38400_i8        12
#define B57600_i8        8

// SPBRG value for FOSC 10 Mhz

#define B2400_10         255
#define B9600_10         64
#define B19200_10        31
#define B57600_10        10

// SPBRG value for FOSC 12 Mhz

#define B2400_12         255
#define B9600_12         78
#define B19200_12        38
#define B57600_12        10


// SPBRG value for FOSC 20 Mhz

#define B9600_20         129
#define B19200_20        64
#define B57600_20        20
// local definition UART.C

#define BUFFER 10                       // rx buffer length
char rx[BUFFER];                        // this is the buffer
unsigned char wpointer,rpointer;        // write and read pointer

#define receivedchar wpointer!=rpointer


// setup PIC UART

void setup_UART(unsigned char diviseur)
{
// set divider for speed
SPBRG=diviseur;
wpointer=1;                             // initialize write pointer
rpointer=1;                             // initialize read pointer
_asm
bcf STATUS,6	
bsf STATUS,5
bsf TXSTA,2				; set BRGH in TXSTA
bcf STATUS,5	
movlw 0x90				
movwf RCSTA				; set RCSTA
bsf STATUS,5				; set TXEN in TXSTA reg	
bsf TXSTA,5
bsf PIE1,5				; set RCIE
bsf INTCON,6				; set PEIE
_endasm;
}	

// rxint is called by interruption service routine

void rxint(void)
{
char caractere;
unsigned char newwp;

     _asm
     bcf STATUS,5
     bcf STATUS,6
     bcf PIR1,5				; clear RCIF
     _endasm;		
     caractere=RCREG;                   // take received char
     if (wpointer!=BUFFER-1)            // if not last place in buffer
	newwp=wpointer+1;               // place=place+1
     else
	newwp=1;                        // else place=1

     if (rpointer!=newwp)               // if read pointer!=write pointer
	rx[wpointer++]=caractere;       // store received char

     if (wpointer==BUFFER)              // if write pointer=length buffer
	 wpointer=1;                    // write pointer = 1
}

// get char

char getch()
{
char caractere=0;

if (receivedchar)
    {
    caractere=rx[rpointer++];
    if (rpointer==BUFFER)
	rpointer=1;
    }
return(caractere);
}

// clear rx buffer

void clear_buffer(void) {
wpointer=1;
rpointer=1;
}

void putc(unsigned char caractere)
{
	_asm
	bsf STATUS,5			; bank1
	btfss TXSTA,1
	goto $-1
	_endasm;
	TXREG=caractere;
}

void print_str(const char* text)
{
	int i=0;
	while (text[i] != 0) {
			putc(text[i]);
			i++;
			}
}

// convert 8 bits binary value and send it on the UART

void print_hex8(unsigned char valeur)
{
  static const char hexa[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
  static unsigned char index;
index=(valeur>>4)& 0x0F;
putc(hexa[index]);
index=valeur&0x0F;
putc(hexa[index]);
}

